@@ -1,13 +1,17 @@
 //! Holds the [Material] struct.
 
+use std::rc::Rc;
+
 use crate::color::Color;
 use crate::light::PointLight;
+use crate::shape::Shape;
+use crate::texture::{solid_color::SolidColor, Texture};
 use crate::tuple::Tuple;
 
 /// Material with lighting properties. Give it to a shape to change its appearance.
 #[derive(Clone)]
 pub struct Material {
-    pub color: Color,
+    pub texture: Rc<dyn Texture>,
     pub ambient: f64,
     pub diffuse: f64,
     pub specular: f64,
@@ -21,13 +25,14 @@ impl Material {
     /// `in_shadow` should be true if `position` is in a shadow of `light`.
     pub fn lighting(
         &self,
+        shape: Rc<dyn Shape>,
         light: &PointLight,
         position: Tuple,
         eye: Tuple,
         normal: Tuple,
         in_shadow: bool,
     ) -> Color {
-        let color = self.color * light.color();
+        let color = self.texture.color_at_shape(position, Rc::clone(&shape)) * light.color();
         let lightv = (light.position() - position).normalized();
         let ambient = color * self.ambient;
         let light_dot_normal = lightv.dot(normal);
@@ -54,7 +59,7 @@ impl Material {
 impl Default for Material {
     fn default() -> Self {
         Self {
-            color: Color::new(1.0, 1.0, 1.0),
+            texture: Rc::new(SolidColor::new(Color::new(1.0, 1.0, 1.0))),
             ambient: 0.1,
             diffuse: 0.9,
             specular: 0.9,
@@ -66,9 +71,12 @@ impl Default for Material {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::shape::sphere::Sphere;
 
     #[test]
     fn lighting_eye_between_light_and_surface() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -76,12 +84,14 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 0.0, -10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, false);
+        let result = material.lighting(shape, &light, position, eye, normal, false);
         assert_eq!(result, Color::new(1.9, 1.9, 1.9));
     }
 
     #[test]
     fn lighting_eye_between_light_and_surface_light_offset_45deg() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -89,12 +99,14 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 0.0, -10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, false);
+        let result = material.lighting(shape, &light, position, eye, normal, false);
         assert_eq!(result, Color::new(1.0, 1.0, 1.0));
     }
 
     #[test]
     fn lighting_eye_opposite_surface_light_offset_45deg() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -102,7 +114,7 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 10.0, -10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, false);
+        let result = material.lighting(shape, &light, position, eye, normal, false);
         assert_eq!(
             result,
             Color::new(0.7363961030678927, 0.7363961030678927, 0.7363961030678927)
@@ -111,6 +123,8 @@ mod tests {
 
     #[test]
     fn lighting_eye_in_path_reflector() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -118,7 +132,7 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 10.0, -10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, false);
+        let result = material.lighting(shape, &light, position, eye, normal, false);
         assert_eq!(
             result,
             Color::new(1.6363961030678928, 1.6363961030678928, 1.6363961030678928)
@@ -127,6 +141,8 @@ mod tests {
 
     #[test]
     fn lighting_light_behind_surface() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -134,12 +150,14 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 0.0, 10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, false);
+        let result = material.lighting(shape, &light, position, eye, normal, false);
         assert_eq!(result, Color::new(0.1, 0.1, 0.1));
     }
 
     #[test]
     fn lighting_surface_in_shadow() {
+        let shape: Rc<dyn Shape> = Rc::new(Sphere::new());
+
         let material = Material::default();
         let position = Tuple::point(0.0, 0.0, 0.0);
 
@@ -147,7 +165,7 @@ mod tests {
         let normal = Tuple::vector(0.0, 0.0, -1.0);
         let light = PointLight::new(Tuple::point(0.0, 0.0, -10.0), Color::new(1.0, 1.0, 1.0));
 
-        let result = material.lighting(&light, position, eye, normal, true);
+        let result = material.lighting(shape, &light, position, eye, normal, true);
         assert_eq!(result, Color::new(0.1, 0.1, 0.1));
     }
 }
